/* UserBaseAlgorithm.java
 *
 * created: Sun Dec 20 1998
 *
 * This file is part of Artemis
 * 
 * Copyright (C) 1998,1999  Genome Research Limited
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * $Header: /nfs/disk222/yeastpub/Repository/powmap/diana/plot/UserBaseAlgorithm.java,v 1.5 1999/05/11 15:19:16 kmr Exp $
 */

package diana.plot;

import uk.ac.sanger.pathogens.*;
import uk.ac.sanger.pathogens.embl.Range;

import diana.sequence.*;

/**
 *  Objects of this class have one useful method - getValues (), which takes a
 *  range of bases and returns a single floating point number.  The return
 *  value is calculated by comparing the bases to the weight matrix that is
 *  passed to constructor.
 *
 *  @author Kim Rutherford
 *  @version $Id: UserBaseAlgorithm.java,v 1.5 1999/05/11 15:19:16 kmr Exp $
 **/

public class UserBaseAlgorithm extends BaseAlgorithm {
  /**
   *  Create a new UserBaseAlgorithm object.  See getValues () for detail on
   *  the weightings array.
   *  @param name The algorithm name.
   *  @param weightings The weight matrix to use in getValues ().  The first
   *    index of this arry is an index from 0 to 3 over the base letters in
   *    this order - T, C, G, A.  The second index is a base position.
   *    All rows of the matrix must be the same length (and at least 1 long).
   *    All columns must be 4 in length (one row for each base).
   **/
  public UserBaseAlgorithm (Strand strand,
                            String name,
                            float [] [] weightings) {
    super (strand, name);

    this.weightings = weightings;
  }

  /**
   *  Return the average value of the user weightings for the sequence of
   *  bases.
   *  An example - a UserBaseAlgorithm created like this:
   *  <pre>
   *   float [] [] weights = { {2,3,4,5}, {1,1,1,1}, {0,1,1,7} };
   *   new UserBaseAlgorithm ("test", weights);
   *  </pre>
   *  with the sequence "ttt" will give (2+1+0)/3 = 1, whereas "gca" will give
   *  (4+1+7)/3 = 4.
   *  @param start The start base (included in the range).
   *  @param end The end base (included in the range).
   *  @param values The one return value for this algorithm is returned in
   *    this array.
   **/
  public void getValues (int start, int end, float [] values) {
    final int number_of_weights = weightings[0].length;

    final String sequence;

    try {
      sequence = getStrand ().getSubSequence (new Range (start, end));
    } catch (OutOfRangeException e) {
      throw new Error ("internal error - unexpected exception: " + e);
    }

    float total = 0;

    for (int i = 0 ; i < sequence.length () ; ++i) {
      final char this_char = sequence.charAt (i);
//      System.out.println (this_char);

      final int this_base_index = Bases.getIndexOfBase (this_char);

      if (this_base_index < 4) {
        total += weightings[this_base_index][i];
      }
    }

//    System.out.println ("total: " + total + " " + sequence);
    
    values[0] = total / number_of_weights;
  }
  
  /**
   *  Return the number of values a call to getValues () will return - one
   *  in this case.
   **/
  public int getValueCount () {
    return 1;
  }

  /**
   *  Return the default or optimal window size.
   *  @return null is returned if this algorithm doesn't have optimal window
   *    size.
   **/
  public Integer getDefaultWindowSize () {
    return new Integer (weightings[0].length);
  } 

  /**
   *  Return the default maximum window size for this algorithm.
   *  @return null is returned if this algorithm doesn't have maximum window
   *    size.
   **/
  public Integer getDefaultMaxWindowSize () {
    return new Integer (weightings[0].length);
  } 

  /**
   *  Return the default minimum window size for this algorithm.
   *  @return null is returned if this algorithm doesn't have minimum window
   *    size.
   **/
  public Integer getDefaultMinWindowSize () {
    return new Integer (weightings[0].length);
  } 

  /**
   *  Return the default or optimal step size.
   *  @return null is returned if this algorithm doesn't have optimal step
   *    size.
   **/
  public Integer getDefaultStepSize (int window_size) {
    return new Integer (1);
  } 

  /**
   *  Return the average value of function over the whole strand.
   *  @return null is returned if this algorithm doesn't have an average or if
   *    the average can't be calculated.
   **/
  public Float getAverage () {
    return null;
  }
  
  /**
   *  The weightings array that was passed to the constructor.
   **/
  float [] [] weightings = null;
}


